@@ -32,13 +32,15 @@ char *__strncpy_null(char *dest, const char *src, size_t n);
 #define strncpy_null(dest, src) __strncpy_null(dest, src, sizeof(dest))
 
 int path_is_block_device(const char *file);
-int path_is_mount_point(const char *file);
+int path_is_a_mount_point(const char *file);
 int path_exists(const char *file);
 int path_is_reg_file(const char *path);
 int path_is_dir(const char *path);
 int is_same_loop_file(const char *a, const char *b);
 int path_is_reg_or_block_device(const char *filename);
 int path_is_in_dir(const char *parent, const char *path);
+char *path_basename(char *path);
+char *path_dirname(char *path);
 
 int test_issubvolname(const char *name);
 
